@@ -1,7 +1,9 @@
+from typing import Union
+
 from algosdk.v2client.algod import AlgodClient
 
 from .asset import Asset, fetch_asset_by_index
-from .pool import Pool, fetch_pool, list_pools
+from .pool import Pool, fetch_pool_by_id, fetch_pools_by_assets, list_pools
 
 
 class PactClient:
@@ -15,18 +17,15 @@ class PactClient:
     def list_pools(self, **kwargs) -> dict:
         return list_pools(self.pact_api_url, **kwargs)
 
-    def fetch_pool(
-        self,
-        primary_asset: Asset,
-        secondary_asset: Asset,
-        app_id=0,
-        fee_bps=30,
-    ) -> Pool:
-        return fetch_pool(
+    def fetch_pools_by_assets(
+        self, primary_asset: Union[Asset, int], secondary_asset: Union[Asset, int]
+    ) -> list[Pool]:
+        return fetch_pools_by_assets(
             algod=self.algod,
             asset_a=primary_asset,
             asset_b=secondary_asset,
             pact_api_url=self.pact_api_url,
-            app_id=app_id,
-            fee_bps=fee_bps,
         )
+
+    def fetch_pool_by_id(self, app_id: int) -> Pool:
+        return fetch_pool_by_id(algod=self.algod, app_id=app_id)
